#!/usr/bin/env python
import sys
import Tkinter
import threading
import Queue
import time
import tkFileDialog
import tkColorChooser
import tkSimpleDialog
import tkMessageBox
import serial
import struct
import pickle
import ConfigParser
import os
import copy

sketch_header = """/* Generated by Pixel Paint */
#include <SPI.h>
#include <TCL.h>
#include <avr/pgmspace.h>

"""
sketch_body = """void setup() {
    TCL.begin();
}

void loop() {
    int i,j;
    byte red;
    byte green;
    byte blue;
    unsigned int addr;

    for(i=0;i<frames;i++) {
        TCL.sendEmptyFrame();
        for(j=0;j<3*frame_size[i];j+=3) {
            addr = pgm_read_word_near(data+i);
            red = pgm_read_byte_near(addr+j);
            green = pgm_read_byte_near(addr+j+1);
            blue = pgm_read_byte_near(addr+j+2);
            TCL.sendColor(red,green,blue);
        }
        TCL.sendEmptyFrame();
        delay(time_delay);
    }
}"""

rootui = None

def main(argv=None):
    global rootui

    if argv is None:
        argv = sys.argv
    
    inputQ = Queue.Queue()
    outputQ = Queue.Queue()
    manager = CommunicationManager(inputQ,outputQ)
    gui = GuiClass(inputQ,outputQ,manager)

    rootui = gui.root

    gui.go()

    return 0

class GuiClass(object):
    def __init__(self,inputQ,outputQ,commManager):
        self.inputQ=inputQ
        self.outputQ=outputQ
        self.manager=commManager

        self.config = ConfigParser.SafeConfigParser()
        self.config.add_section('canvas')
        self.config.add_section('grid')
        self.config.add_section('lights')
        self.config.add_section('files')
        self.config.set('canvas','height','350')
        self.config.set('canvas','width','350')
        self.config.set('grid','nx','5')
        self.config.set('grid','ny','5')
        self.config.set('lights','filename','')
        default_directory = ''
        if sys.platform=='darwin':
            default_directory = os.path.expanduser('~/Documents')
        elif sys.platform=='win32':
            default_directory = os.path.expanduser('~/My Documents')
        else:
            default_directory = '.'
        self.config.set('files','default_directory',default_directory)
        self.config.set('files','arduino_directory',default_directory)
        self.config.read(os.path.expanduser('~/.pixel-paint.cfg'))

        self.root = Tkinter.Tk()
        self.root.protocol("WM_DELETE_WINDOW", self.quit) # Save config state before quitting
        
        canvas_frame = Tkinter.Frame(self.root)
        entry_frame = Tkinter.Frame(self.root)
        button_frame = Tkinter.Frame(self.root)
        
        self.canvas = Tkinter.Canvas(canvas_frame,height=self.config.getint('canvas','height'),width=self.config.getint('canvas','width'))
        self.canvas.pack(fill=Tkinter.BOTH,expand=True)
        self.canvas.bind('<Button-1>',self.ovalClick)

        self.color = Color(0xff,0x00,0x00)
        self.pixelShow = PixelShow()
        self.frames = self.pixelShow.getFrames()
        self.frames.append(ColorGrid(self.config.getint('grid','nx'),self.config.getint('grid','ny')))
        self.currentFrame = Tkinter.IntVar()
        self.currentFrame.set(0)
        self.colorGrid = self.frames[0]

        frameLabel = Tkinter.Label(entry_frame, text="Frame Number: ")
        frameValue = Tkinter.Label(entry_frame, width=10, textvariable=self.currentFrame,anchor=Tkinter.W,justify=Tkinter.LEFT)
        prevFrameButton = Tkinter.Button(entry_frame,text="Previous Frame",command=self.prevFrame)
        nextFrameButton = Tkinter.Button(entry_frame,text="Next Frame",command=self.nextFrame)
        frameLabel.pack(side=Tkinter.LEFT)
        frameValue.pack(side=Tkinter.LEFT)
        prevFrameButton.pack(side=Tkinter.LEFT)
        nextFrameButton.pack(side=Tkinter.LEFT)

        resetButton = Tkinter.Button(button_frame,text="Reset",command=self.reset)
        colorSelectButton = Tkinter.Button(button_frame,text="Color Select",command=self.selectColor)
        quitButton = Tkinter.Button(button_frame,text="Quit",command=self.quit)
        resetButton.pack(side=Tkinter.LEFT,fill=Tkinter.X)
        colorSelectButton.pack(side=Tkinter.LEFT,fill=Tkinter.X)
        quitButton.pack(side=Tkinter.LEFT,fill=Tkinter.X)
        button_frame.pack(fill=Tkinter.X)
        entry_frame.pack(fill=Tkinter.X)
        canvas_frame.pack(fill=Tkinter.BOTH,expand=True)
        quitButton.pack(fill=Tkinter.X)
        resetButton.pack(fill=Tkinter.X)
        colorSelectButton.pack(fill=Tkinter.X)

        self.root.bind("<Configure>",self.setupCanvas)

        # Menubar
        menubar = Tkinter.Menu(self.root)
        filemenu = Tkinter.Menu(menubar,tearoff=False)
        lightmenu = Tkinter.Menu(menubar,tearoff=False)
        showmenu = Tkinter.Menu(menubar,tearoff=False)
        if sys.platform=='darwin':
            apple = Tkinter.Menu(menubar,tearoff=False, name='apple')
        else:
            apple = Tkinter.Menu(menubar,tearoff=False)
        lightmenu.add_command(label="Connect...",command=self.connect)
        lightmenu.add_command(label="Disconnect",command=self.disconnect)
        lightmenu.add_separator()
        lightmenu.add_command(label="Create Sketch",command=self.createSketch)
        filemenu.add_command(label="Load Image",command=self.loadImage)
        filemenu.add_command(label="Save Image",command=self.saveImage)
        filemenu.add_separator()
        filemenu.add_command(label="Load Show",command=self.loadShow)
        filemenu.add_command(label="Save Show",command=self.saveShow)
        filemenu.add_separator()
        filemenu.add_command(label="Quit",command=self.quit)
        showmenu.add_command(label="New Show",command=self.newShow)
        showmenu.add_command(label="Change Pixel Grid",command=self.newImage)
        showmenu.add_command(label="Set Frame Delay",command=self.setFrameDelay)
        showmenu.add_separator()
        showmenu.add_command(label="Add Blank Frame",command=self.addFrame)
        showmenu.add_command(label="Add Copied Frame",command=self.addFrameCopy)
        showmenu.add_command(label="Delete Frame",command=self.deleteFrame)
        # showmenu.add_command(label="Create Sequence",command=self.createSequence)
        showmenu.add_separator()
        showmenu.add_command(label="Play Show",command=self.playShow)
        showmenu.add_command(label="Stop Show",command=self.stopShow)
        apple.add_command(label="About PixelPaint",command=self.about)
        if sys.platform=='darwin':
            menubar.add_cascade(menu=apple)
        menubar.add_cascade(label="File",menu=filemenu)
        menubar.add_cascade(label="Arduino",menu=lightmenu)
        menubar.add_cascade(label="Images",menu=showmenu)
        if sys.platform!='darwin':
            menubar.add_cascade(label="Help",menu=apple)
        self.root.config(menu=menubar)
        self.root.title("Pixel-Paint")

    def ovalClick(self,event):
        x=self.canvas.canvasx(event.x)
        y=self.canvas.canvasy(event.y)
        w=self.canvas.find_closest(x,y)
        i,j = self.ovalDict[w[0]]
        self.canvas.itemconfig(w,fill=self.color.toWidgetColor())
        self.colorGrid.put(i,j,self.color)
        self.updateLEDs()

    def selectColor(self):
        colorChoice = tkColorChooser.askcolor(self.color.toWidgetColor())
        if colorChoice[0]:
            self.color = Color(colorChoice[0][0],colorChoice[0][1],colorChoice[0][2])

    def updateLEDs(self):
        if self.manager.isConnected():
            datastring = self.colorGrid.returnSequence(self.corner,self.orientation)
            self.outputQ.put("DATA %d\n"%len(datastring))
            self.outputQ.put(''.join(datastring))

    def saveImage(self):
        default_directory = self.config.get('files','default_directory')
        filename = tkFileDialog.asksaveasfilename(defaultextension=".led",filetypes=[("LED File","*.led"),("All Files","*")],initialdir=default_directory)
        if filename == '':
            return
        head, tail = os.path.split(filename)
        root, ext = os.path.splitext(filename)
        self.config.set('files','default_directory',head)
        if ext=='':
            filename = filename+".led"
        f = open(filename,'wb')
        pickle.dump(self.colorGrid,f)
        f.close()

    def loadImage(self):
        default_directory = self.config.get('files','default_directory')
        filename = tkFileDialog.askopenfilename(defaultextension=".led",filetypes=[("LED File","*.led"),("All Files","*")],initialdir=default_directory)
        if filename == '':
            return
        f = open(filename,'rb')
        self.colorGrid = pickle.load(f)
        f.close()
        head, tail = os.path.split(filename)
        self.config.set('files','default_directory',head)
        fnum = self.currentFrame.get()
        self.frames[fnum] = self.colorGrid
        self.updateFrame()

    def newImage(self):
        d = SizeDialog(self.root)
        if d.result:
            self.colorGrid = ColorGrid(d.result[0],d.result[1])
            fnum = self.currentFrame.get()
            self.frames[fnum] = self.colorGrid
            self.setupCanvas()

    def go(self):
        self.root.after(100, self.periodic_check)
        self.root.mainloop()

    def connect(self):
        d = ConnectionDialog(self.root,self.config.get('lights','filename'))
        if d.result:
            self.corner,self.orientation,filename = d.result
            self.config.set('lights','filename',filename)
            self.manager.connect(filename)
            self.root.after(2000,self.check_connection)

    def check_connection(self):
        self.outputQ.put("CHECK\n")

    def periodic_check(self):
        try:
            inp = self.inputQ.get_nowait()
            tokens = inp.split()
            if len(tokens)>=1:
                if tokens[0]=="CONNECT":
                    # print "Arduino Connected"
                    self.updateLEDs()
                elif tokens[0]=="ERROR":
                    tkMessageBox.showwarning("Warning","Error from Arduino: "+' '.join(tokens[1:]),parent=self.root)
                    # print "Error: "+' '.join(tokens[1:])
                else:
                    tkMessageBox.showwarning("Warning","Unrecognized response from Arduino: %s"%inp,parent=self.root)
                    # print "Unrecognized response:"
                    # print inp
        except Queue.Empty:
            pass
            # self.writeline("No Data")
        self.root.after(100, self.periodic_check)

    def reset(self):
        for j in range(self.colorGrid.getNX()):
            for i in range(self.colorGrid.getNY()):
                self.colorGrid.put(j,i,Color(0,0,0))

        for w in self.ovalDict.keys():
            self.canvas.itemconfig(w,fill="black")

        self.updateLEDs()

    def setupCanvas(self,event=None):
        self.canvas.delete(Tkinter.ALL)
        width = self.canvas.winfo_width()
        height = self.canvas.winfo_height()

        nx = self.colorGrid.getNX()
        ny = self.colorGrid.getNY()
        spacex = width/nx
        spacey = height/ny
        spacing = spacex
        if spacing>spacey:
            spacing = spacey

        self.ovalDict = {}
        for j in range(ny):
            for i in range(nx):
                w = self.canvas.create_oval(spacing*i,spacing*j,spacing*(i+1),spacing*(j+1),fill=self.colorGrid.get(i,j).toWidgetColor(),activeoutline="red")
                self.ovalDict[w]=(i,j)

    def disconnect(self):
        self.manager.close()

    def playShow(self):
        self.playSequence = list(self.pixelShow.getSequence())
        self.keepPlaying=True
        if len(self.playSequence)==0:
            self.playSequence = range(len(self.frames))
        self.playNext()

    def playNext(self):
        if len(self.playSequence) and self.keepPlaying:
            fnum = self.playSequence.pop(0)
            self.colorGrid = self.frames[fnum]
            self.currentFrame.set(fnum)
            self.updateFrame()
            self.root.after(self.pixelShow.getDelay(),self.playNext)
        elif self.keepPlaying:
            self.playShow()

    def stopShow(self):
        self.keepPlaying=False

    def loadShow(self):
        default_directory = self.config.get('files','default_directory')
        filename = tkFileDialog.askopenfilename(defaultextension=".sho",filetypes=[("SHOW File","*.sho"),("All Files","*")],initialdir=default_directory)

        if filename == '':
            return
        f = open(filename,'rb')
        self.pixelShow = pickle.load(f)
        f.close()
        head, tail = os.path.split(filename)
        self.config.set('files','default_directory',head)
        self.currentFrame.set(0)
        self.frames = self.pixelShow.getFrames()
        self.colorGrid = self.frames[0]
        self.updateFrame()

    def saveShow(self):
        default_directory = self.config.get('files','default_directory')
        filename = tkFileDialog.asksaveasfilename(defaultextension=".sho",filetypes=[("SHOW File","*.sho"),("All Files","*")],initialdir=default_directory)

        if filename == '':
            return
        head, tail = os.path.split(filename)
        root, ext = os.path.splitext(filename)
        self.config.set('files','default_directory',head)
        if ext=='':
            filename = filename+".sho"
        f = open(filename,'wb')
        pickle.dump(self.pixelShow,f)
        f.close()

    def newShow(self):
        self.pixelShow = PixelShow()
        self.frames = self.pixelShow.getFrames()
        self.frames.append(self.colorGrid)
        self.currentFrame.set(0)

    def setFrameDelay(self):
        delay = tkSimpleDialog.askinteger("Frame Delay","Delay between frames (ms):")
        self.pixelShow.setDelay(delay)

    def nextFrame(self):
        fnum = self.currentFrame.get()
        fnum = fnum+1
        if fnum==len(self.frames):
            fnum=0

        self.currentFrame.set(fnum)
        self.colorGrid = self.frames[fnum]
        self.updateFrame()

    def prevFrame(self):
        fnum = self.currentFrame.get()
        fnum = fnum-1
        if fnum==-1:
            fnum = len(self.frames)-1

        self.currentFrame.set(fnum)
        self.colorGrid = self.frames[fnum]
        self.updateFrame()

    def addFrame(self):
        fnum = self.currentFrame.get()
        fnum = fnum+1
        self.frames.insert(fnum,ColorGrid(self.colorGrid.getNX(),self.colorGrid.getNY()))
        self.colorGrid = self.frames[fnum]
        self.currentFrame.set(fnum)
        self.updateFrame()

    def addFrameCopy(self):
        fnum = self.currentFrame.get()
        fnum = fnum+1
        self.frames.insert(fnum,copy.deepcopy(self.colorGrid))
        self.colorGrid = self.frames[fnum]
        self.currentFrame.set(fnum)
        self.updateFrame()

    def updateFrame(self):
        self.setupCanvas()
        for w in self.ovalDict.keys():
            i, j = self.ovalDict[w]
            self.canvas.itemconfig(w,fill=self.colorGrid.get(i,j).toWidgetColor())
        self.updateLEDs()

    def deleteFrame(self):
        fnum = self.currentFrame.get()
        del self.frames[fnum]
        if len(self.frames)==0:
            self.addFrame()
        else:
            self.prevFrame()

    def createSequence(self):
        pass

    def createSketch(self):
        d = SketchDialog(self.root)
        if d.result:
            corner,orientation = d.result
        else:
            return

        default_directory = self.config.get('files','arduino_directory')
        filename = tkFileDialog.asksaveasfilename(defaultextension=".ino",filetypes=[("Arduino Sketch","*.ino"),("All Files","*")],initialdir=default_directory)

        if filename == '':
            return

        head, tail = os.path.split(filename)
        root, ext = os.path.splitext(filename)
        self.config.set('files','arduino_directory',head)
        if ext=='':
            filename = filename+".ino"

        f = open(filename,'w')
        f.write(sketch_header)
        f.write(self.pixelShow.getArduinoSketch(corner,orientation))
        f.write(sketch_body)
        f.close()
        return

    def about(self):
        aboutText = u"""PixelPaint Version 0.4

Copyright \u00a9 2012 Christopher De Vries

PixelPaint is made available subject to the terms of the Artistic License 2.0 a copy of which can be found at http://opensource.org/licenses/Artistic-2.0."""

        tkMessageBox.showinfo("About",aboutText,parent=self.root)
        return

    def quit(self):
        self.config.set('canvas','height',str(self.canvas.winfo_height()))
        self.config.set('canvas','width',str(self.canvas.winfo_width()))
        self.config.set('grid','nx',str(self.colorGrid.getNX()))
        self.config.set('grid','ny',str(self.colorGrid.getNY()))
        with open(os.path.expanduser('~/.pixel-paint.cfg'),'w') as configfile:
            self.config.write(configfile)
        self.root.destroy() 

class CommunicationManager(object):
    def __init__(self,inputQ,outputQ):
        self.inputQ=inputQ
        self.outputQ=outputQ
        self.serialPort=None
        self.inputThread = None
        self.outputThread = None
        self.keepRunning = True
        self.activeConnection = False

    def runInput(self):
        while self.keepRunning:
            try:
                inputline = self.serialPort.readline()
                self.inputQ.put(inputline.rstrip())
            except:
                pass
                # print "IO Error - Input"
        return

    def runOutput(self):
        while self.keepRunning:
            try:
                outputline = self.outputQ.get()
                self.serialPort.write("%s"%outputline)
            except:
                pass
                # print "IO Error - Output"

        return

    def connect(self,filename):
        global rootui

        self.close()
        try:
            self.serialPort = serial.Serial(filename,115200)
        except serial.SerialException:
            tkMessageBox.showwarning("Connection","Unable to connect to serial port %s"%filename,parent=rootui)
            # self.inputQ.put("ERROR Unable to Connect to Serial Port")
            return
        self.keepRunning=True
        self.inputThread = threading.Thread(target=self.runInput)
        self.inputThread.daemon=True
        self.inputThread.start()
        self.outputThread = threading.Thread(target=self.runOutput)
        self.outputThread.daemon=True
        self.outputThread.start()
        self.activeConnection = True

    def close(self):
        if self.activeConnection:
            self.keepRunning=False
            self.serialPort.close()
            self.outputQ.put("TERMINATE")
            self.inputThread.join()
            self.outputThread.join()
            # print "All IO Threads stopped"
            self.activeConnection = False

    def isConnected(self):
        """Is there a connection?"""
        return self.activeConnection

class Color(object):
    def __init__(self,red,green,blue):
        self.red=red
        self.green=green
        self.blue=blue

    def toWidgetColor(self):
        return "#%02x%02x%02x"%(self.red,self.green,self.blue)

    def toByteSequence(self):
        rgb = struct.pack(">BBB",self.red,self.green,self.blue)

        return rgb

    def toTextSequence(self):
        return "%#x,%#x,%#x"%(self.red,self.green,self.blue)

class ColorGrid(object):
    BL = 0
    BR = 1
    TL = 2
    TR = 3
    HORIZONTAL = 0
    VERTICAL = 1

    def __init__(self,nx,ny):
        self.nx=nx
        self.ny=ny
        self.colorGrid = []

        for j in range(ny):
            self.colorGrid.append([])
            for i in range(nx):
                self.colorGrid[j].append(Color(0,0,0))
    
    def get(self,x,y):
        return self.colorGrid[y][x]

    def put(self,x,y,color):
        self.colorGrid[y][x]=color

    def getNX(self):
        return self.nx

    def getNY(self):
        return self.ny

    def returnColorSequence(self,start=BL,direction=HORIZONTAL):
        colorsequence = []
        
        xseq = range(self.nx)
        yseq = range(self.ny)
        if start==ColorGrid.TL:
            pass
        elif start==ColorGrid.TR:
            xseq.reverse()
        elif start==ColorGrid.BL:
            yseq.reverse()
        else:
            xseq.reverse()
            yseq.reverse()

        if direction==ColorGrid.HORIZONTAL:
            for j in yseq:
                for i in xseq:
                    colorsequence.append(self.colorGrid[j][i])
                xseq.reverse()

        if direction==ColorGrid.VERTICAL:
            for i in xseq:
                for j in yseq:
                    colorsequence.append(self.colorGrid[j][i])
                yseq.reverse()

        return colorsequence

    def returnSequence(self,start=BL,direction=HORIZONTAL):
        colorsequence = self.returnColorSequence(start,direction)
        datastring = [color.toByteSequence() for color in colorsequence]

        return datastring

    def returnSketchSequence(self,start=BL,direction=HORIZONTAL):
        colorsequence = self.returnColorSequence(start,direction)
        datastring = [color.toTextSequence() for color in colorsequence]

        return datastring

class PixelShow(object):
    """Pixel Show.

    A pixel show contains frames of data displayed in a sequence with a delay
    between frames. The frames are stored as ColorGrids in the frame array
    and the sequence is a sequence of integers referring to the frames in the
    frame array."""
    def __init__(self,delay=1000):
        """Pixel show, delay is delay between frames in milliseconds"""
        self.delay=delay
        self.frames = []
        self.sequence = []

    def getFrames(self):
        return self.frames

    def getSequence(self):
        return self.sequence

    def setDelay(self,delay):
        self.delay=delay

    def getDelay(self):
        return self.delay

    def getArduinoSketch(self,start=ColorGrid.BL,direction=ColorGrid.HORIZONTAL):
        stringbuffer = []
        frame_sizes = []
        frame_num = 0
        for frame in self.frames:
            sequence = frame.returnSketchSequence(start,direction)
            stringbuffer.append("PROGMEM prog_uchar frame%d[] = {"%frame_num)
            stringbuffer.append(','.join(sequence))
            stringbuffer.append("};\n")
            frame_sizes.append(str(len(sequence)))
            frame_num = frame_num+1

        stringbuffer.append("int frame_size[] = {")
        stringbuffer.append(','.join(frame_sizes))
        stringbuffer.append("};\n")

        stringbuffer.append("int frames=%d;\n"%frame_num)

        stringbuffer.append("PROGMEM const prog_uchar *data[] = {");
        frame_names = ["frame%d"%i for i in range(frame_num)]
        stringbuffer.append(','.join(frame_names))
        stringbuffer.append("};\n")
        stringbuffer.append("int time_delay=%d;\n"%self.delay)

        return ''.join(stringbuffer)

class SizeDialog(tkSimpleDialog.Dialog):
    """This class creates a new popup that asks the size of the new image"""
    def body(self,master):
        lx = Tkinter.Label(master,text="Horizonal Pixels:")
        ly = Tkinter.Label(master,text="Vertical Pixels:")
        self.nx = Tkinter.IntVar()
        self.ny = Tkinter.IntVar()
        ex = Tkinter.Entry(master,width=10,textvariable=self.nx)
        ey = Tkinter.Entry(master,width=10,textvariable=self.ny)

        lx.pack(side=Tkinter.LEFT)
        ex.pack(side=Tkinter.LEFT)
        ly.pack(side=Tkinter.LEFT)
        ey.pack(side=Tkinter.LEFT)

    def apply(self):
        self.result = self.nx.get(),self.ny.get()

class ConnectionDialog(tkSimpleDialog.Dialog):
    """This dialog gets the parameters to connect with the Arduino"""
    def __init__(self,master,default_filename,title=None):
        self.default_filename = default_filename
        tkSimpleDialog.Dialog.__init__(self,master,title)

    def body(self,master):
        cornerFrame = Tkinter.Frame(master)
        hvFrame = Tkinter.Frame(master)
        fileFrame = Tkinter.Frame(master)

        self.corner=Tkinter.IntVar(0)
        self.orientation=Tkinter.IntVar(0)
        self.arduinoFile = Tkinter.StringVar()
        self.arduinoFile.set(self.default_filename)
        
        blButton = Tkinter.Radiobutton(cornerFrame,text="Bottom Left",variable=self.corner,value=ColorGrid.BL)
        brButton = Tkinter.Radiobutton(cornerFrame,text="Bottom Right",variable=self.corner,value=ColorGrid.BR)
        tlButton = Tkinter.Radiobutton(cornerFrame,text="Top Left",variable=self.corner,value=ColorGrid.TL)
        trButton = Tkinter.Radiobutton(cornerFrame,text="Top Right",variable=self.corner,value=ColorGrid.TR)

        horButton = Tkinter.Radiobutton(hvFrame,text="Horizontal",variable=self.orientation,value=ColorGrid.HORIZONTAL)
        verButton = Tkinter.Radiobutton(hvFrame,text="Vertical",variable=self.orientation,value=ColorGrid.VERTICAL)

        cornerLabel=Tkinter.Label(cornerFrame,text="Corner of First Pixel")
        orientationLabel=Tkinter.Label(hvFrame,text="Orientation of Pixels")

        portLabel = Tkinter.Label(fileFrame,text="Arduino Port: ")
        portEntry = Tkinter.Entry(fileFrame,width=30,textvariable=self.arduinoFile)

        cornerLabel.pack(anchor=Tkinter.W)
        blButton.pack(anchor=Tkinter.W)
        brButton.pack(anchor=Tkinter.W)
        tlButton.pack(anchor=Tkinter.W)
        trButton.pack(anchor=Tkinter.W)

        orientationLabel.pack(anchor=Tkinter.W)
        horButton.pack(anchor=Tkinter.W)
        verButton.pack(anchor=Tkinter.W)

        portLabel.pack(side=Tkinter.LEFT)
        portEntry.pack(side=Tkinter.LEFT,fill=Tkinter.X,expand=True)

        fileFrame.pack(fill=Tkinter.X,expand=True)
        cornerFrame.pack(side=Tkinter.LEFT)
        hvFrame.pack(side=Tkinter.LEFT)

    def apply(self):
        self.result = self.corner.get(),self.orientation.get(),self.arduinoFile.get()

class SketchDialog(tkSimpleDialog.Dialog):
    """This dialog gets the parameters to connect with the Arduino"""
    def body(self,master):
        cornerFrame = Tkinter.Frame(master)
        hvFrame = Tkinter.Frame(master)
        
        self.corner=Tkinter.IntVar(0)
        self.orientation=Tkinter.IntVar(0)
        
        blButton = Tkinter.Radiobutton(cornerFrame,text="Bottom Left",variable=self.corner,value=ColorGrid.BL)
        brButton = Tkinter.Radiobutton(cornerFrame,text="Bottom Right",variable=self.corner,value=ColorGrid.BR)
        tlButton = Tkinter.Radiobutton(cornerFrame,text="Top Left",variable=self.corner,value=ColorGrid.TL)
        trButton = Tkinter.Radiobutton(cornerFrame,text="Top Right",variable=self.corner,value=ColorGrid.TR)

        horButton = Tkinter.Radiobutton(hvFrame,text="Horizontal",variable=self.orientation,value=ColorGrid.HORIZONTAL)
        verButton = Tkinter.Radiobutton(hvFrame,text="Vertical",variable=self.orientation,value=ColorGrid.VERTICAL)

        cornerLabel=Tkinter.Label(cornerFrame,text="Corner of First Pixel")
        orientationLabel=Tkinter.Label(hvFrame,text="Orientation of Pixels")

        cornerLabel.pack(anchor=Tkinter.W)
        blButton.pack(anchor=Tkinter.W)
        brButton.pack(anchor=Tkinter.W)
        tlButton.pack(anchor=Tkinter.W)
        trButton.pack(anchor=Tkinter.W)

        orientationLabel.pack(anchor=Tkinter.W)
        horButton.pack(anchor=Tkinter.W)
        verButton.pack(anchor=Tkinter.W)

        cornerFrame.pack(side=Tkinter.LEFT)
        hvFrame.pack(side=Tkinter.LEFT)

    def apply(self):
        self.result = self.corner.get(),self.orientation.get()

if __name__ == "__main__":
    sys.exit(main())
